#!/usr/bin/env python

"""
Webware classes.
"""

import Generic
from cgi import parse_qs
import StringIO
from Helpers import Environment
from Helpers.Request import Cookie

class Transaction(Generic.Transaction):

    """
    Webware transaction interface.
    """

    def __init__(self, trans):

        "Initialise the transaction using the Webware transaction 'trans'."

        self.trans = trans
        self.user = None

    # Request-related methods.

    def get_request_stream(self):

        """
        A framework-specific method which returns the request stream for
        the transaction.
        """

        request = self.trans.request()
        try:
            stream = request.rawInput(rewind=1)
            if stream is None:
                return StringIO.StringIO("")

        # NOTE: Dubious catch-all situation, but it is difficult to control
        # NOTE: cases where Webware's internals themselves fail.

        except:
            return StringIO.StringIO("")

        return stream

    def get_request_method(self):

        """
        A framework-specific method which gets the request method.
        """

        return self.trans.request().method()

    def get_headers(self):

        """
        A framework-specific method which returns all request headers as a
        dictionary-like object mapping header names to values.
        NOTE: If duplicate header names are permitted, then this interface will
        NOTE: need to change.
        """

        # Use the Webware environment and some assumptions about variable names.
        # NOTE: Using lower case for the header names.

        env = self.trans.request().environ()
        return Environment.get_headers(env)

    def get_header_values(self, key):

        """
        A framework-specific method which returns a list of all request header
        values associated with the given 'key'. Note that according to RFC 2616,
        'key' is treated as a case-insensitive string.
        """

        # Use the Webware environment and some assumptions about variable names.

        env = self.trans.request().environ()
        cgi_key = "HTTP_" + key.replace("-", "_").upper()
        if env.has_key(cgi_key):
            return [env[cgi_key]]
        else:
            return []

    def get_content_type(self):

        """
        A framework-specific method which gets the content type specified on the
        request, along with the charset employed.
        """

        return self.parse_content_type(self.trans.request().contentType())

    def get_content_charsets(self):

        """
        Returns the character set preferences.
        NOTE: Requires enhancements to HTTPRequest.
        """

        return self.trans.request().contentCharsets()

    def get_content_languages(self):

        """
        A framework-specific method which extracts language information from
        the transaction.
        NOTE: Requires enhancements to HTTPRequest.
        """

        return self.trans.request().contentLanguages()

    def get_path(self):

        """
        A framework-specific method which gets the entire path from the request.
        """

        return self.trans.request().uri()

    def get_path_info(self):

        """
        A framework-specific method which gets the "path info" (the part of the
        URL after the resource name handling the current request) from the
        request.
        """

        return self.trans.request().extraURLPath()

    def get_query_string(self):

        """
        A framework-specific method which gets the query string from the path in
        the request.
        """

        return self.trans.request().queryString()

    # Higher level request-related methods.

    def get_fields_from_path(self):

        """
        A framework-specific method which extracts the form fields from the
        path specified in the transaction. The underlying framework may refuse
        to supply fields from the path if handling a POST transaction.

        Returns a dictionary mapping field names to lists of values (even if a
        single value is associated with any given field name).
        """

        return parse_qs(self.get_query_string(), keep_blank_values=1)

    def get_fields_from_body(self):

        """
        A framework-specific method which extracts the form fields from the
        message body in the transaction.

        Returns a dictionary mapping field names to lists of values (even if a
        single value is associated with any given field name).
        """

        # Fix the non-list results.

        fields = {}
        for field_name, field_value in self.trans.request().fields().items():
            if type(field_value) == type([]):
                fields[field_name] = field_value
            else:
                fields[field_name] = [field_value]
        return fields

    def get_user(self):

        """
        A framework-specific method which extracts user information from the
        transaction.

        Returns a username as a string or None if no user is defined.
        """

        # NOTE: Webware relies entirely on a CGI-style environment where the
        # NOTE: actual headers are not available. Therefore, the Web server must
        # NOTE: itself be set up to provide user support.

        if self.user is not None:
            return self.user

        try:
            return self.trans.request().remoteUser()
        except KeyError, exc:
            return None

    def get_cookies(self):

        """
        A framework-specific method which obtains cookie information from the
        request.

        Returns a dictionary mapping cookie names to cookie objects.
        """

        cookies = {}
        for name, value in self.trans.request().cookies().items():
            cookies[name] = Cookie(name, value)
        return cookies

    def get_cookie(self, cookie_name):

        """
        A framework-specific method which obtains cookie information from the
        request.

        Returns a cookie object for the given 'cookie_name' or None if no such
        cookie exists.
        """

        try:
            return Cookie(cookie_name, self.trans.request().cookie(cookie_name))
        except KeyError:
            return None

    # Response-related methods.

    def get_response_stream(self):

        """
        A framework-specific method which returns the response stream for
        the transaction.
        """

        return self.trans.response()

    def get_response_code(self):

        """
        Get the response code associated with the transaction. If no response
        code is defined, None is returned.
        """

        # NOTE: Webware treats the response code as just another header.

        status = self.trans.response().header("Status", None)
        try:
            if status is not None:
                return int(status)
            else:
                return None
        except ValueError:
            return None

    def set_response_code(self, response_code):

        """
        Set the 'response_code' using a numeric constant defined in the HTTP
        specification.
        """

        self.trans.response().setStatus(response_code)

    def set_header_value(self, header, value):

        """
        Set the HTTP 'header' with the given 'value'.
        """

        self.trans.response().setHeader(self.format_header_value(header), self.format_header_value(value))

    def set_content_type(self, content_type):

        """
        A framework-specific method which sets the 'content_type' for the
        response.
        """

        return self.trans.response().setHeader("Content-Type", self.format_content_type(content_type))

    # Higher level response-related methods.

    def set_cookie(self, cookie):

        """
        A framework-specific method which stores the given 'cookie' object in
        the response.
        """

        self.trans.response().addCookie(cookie)

    def set_cookie_value(self, name, value, path=None, expires=None):

        """
        A framework-specific method which stores a cookie with the given 'name'
        and 'value' in the response.

        The optional 'path' is a string which specifies the scope of the cookie,
        and the optional 'expires' parameter is a value compatible with the
        time.time function, and indicates the expiry date/time of the cookie.
        """

        self.trans.response().setCookie(name, value, path, expires)

    def delete_cookie(self, cookie_name):

        """
        A framework-specific method which adds to the response a request that
        the cookie with the given 'cookie_name' be deleted/discarded by the
        client.
        """

        self.trans.response().delCookie(cookie_name)

    # Application-specific methods.

    def set_user(self, username):

        """
        An application-specific method which sets the user information with
        'username' in the transaction. This affects subsequent calls to
        'get_user'.
        """

        self.user = username

# vim: tabstop=4 expandtab shiftwidth=4
